# ITC for GRACES.
# Version beta, released 20150918 by ANC.
# Version 1.0, released 20150922 by ANC.
# Version 1.1, released 20160208 by ANC. - now can apply reddening to the template
# Version 1.2, released 20160421 by ANC. - cuts the template spectrum between 350nm and 1250nm; the iterator for the loop creating the .dat file is now in format long
# Version 1.3, released 20160823 by ANC. - gets rid of some of the output curves to avoid confusion in the ITC results
# Version 1.4, released 20161006 by KC. - fixes problems with M giant templates
# Version 2.0, released 20200520 by TM. - IDL code rewritten in Python; ITC recalibrated; several bug fixes; new features added
# Version 2.1, released 20200816 by TM. - UBVRI filter options added; refined photometric zeropoints for all filters; overheads added

import textwrap
import os
import argparse
import numpy as np
from scipy.interpolate import interp1d
import matplotlib.pyplot as plt

# Setting up arguments and help
epilog = "Example:\n$ python ITC.py -templ g2v -mag 10 -filter r -iq 85 -cc 70 -sb 80 -airmass 1.5 -rdmode normal -spmode 1 -exptime 120\n\nOnce the script is done, it displays and saves a .png plot of the estimated S/N vs. wavelength and a corresponding .txt file, which are both saved in the dirout directory. It also prints lines with useful information. Please copy and paste them in your proposal right before attaching the plot.\n\nThe central wavelength at which the guiding is optimized is assumed to be 700nm. If shorter central wavelength is selected, then the blue efficiency will be enhanced at the expense of the redder wavelengths due to the atmospheric differential refraction, and vice versa for the longer central wavelength. The differential refraction is stronger at higher airmasses and negligible near zenith.\n\nIf you have any questions or requests, please contact the GRACES team:\nhttps://www.gemini.edu/instrumentation/current-instruments/graces"
wrapping = []
for line in epilog.splitlines():
	wrapping.append(textwrap.fill(line, width=os.get_terminal_size()[0]))
epilog = "\n".join(wrapping)

parser = argparse.ArgumentParser(description="*** Help for GRACES ITC v2.1 ***", epilog=epilog, formatter_class=lambda prog:argparse.RawDescriptionHelpFormatter(prog, width=os.get_terminal_size()[0]))

parser.add_argument("-dirin", type=str, metavar="str", help="Path where the folders \"filters\" and \"templates\" are. Default is the current working directory.")
parser.add_argument("-dirout", type=str, metavar="str", help="Path where you wish to save the output files. Default is the current working directory.")
parser.add_argument("-name", type=str, metavar="str", help="Basename of the output files. Default is \"ITC\".")
parser.add_argument("-objtype", type=str, metavar="str", help="Spatial brightness profile. Options are \"pointsource\", \"gaussian\", and \"uniform\". Default is \"pointsource\".")
parser.add_argument("-fwhm", type=float, metavar="float", help="FWHM of the Gaussian brightness profile in arcsec. This parameter is only used when \"gaussian\" objtype is selected.")
parser.add_argument("-templ", type=str, metavar="str", help="Spectral template. It has to be written exactly as expected by the script. For example, a spectral type A0III needs to be written \"a0iii\", and a planetary nebula is \"planetary-nebula\". Upper cases are accepted as well. If you cannot guess the exact spelling, either check the templates folder, or omit the templ input and you will be provided with a list of choices. If you want to use a custom template, set templ to \"custom\". In this case, you are expected to provide the filename of your custom template spectrum, either as an input or once a prompt is requesting it. Note that \"qso-at-z\" requires the redshift, \"blackbody\" requires the temperature, and \"powerlaw\" requires the exponent input.")
parser.add_argument("-ctempl", type=str, metavar="str", help="Path and filename of your custom spectral template. The file has to be in ASCII format. The script expects two columns separated by a whitespace. The first column should be wavelength (in nm), and the second column flux (any unit). If path is omited, the file is assumed to be the current working directory.")
parser.add_argument("-temp", type=float, metavar="float", help="Temperature for the \"blackbody\" template in K.")
parser.add_argument("-expn", type=float, metavar="float", help="Exponent for the \"powerlaw\" template.")
parser.add_argument("-mag", type=float, metavar="float", help="Magnitude of the \"pointsource\" target, spatially integrated magnitude of the \"gaussian\" target, or surface brightness in mag/arcsec^2 of the \"uniform\" target.")
parser.add_argument("-filter", type=str, metavar="str", help="Filter in which the magnitude is given (UBVRIugri).")
parser.add_argument("-z", type=float, metavar="float", help="Redshift. Default is 0.")
parser.add_argument("-av", type=float, metavar="float", help="Extinction in V mag. Default is 0.")
parser.add_argument("-rv", type=float, metavar="float", help="Rv=Av/E(B-V). This parameter is only used when Av is defined and nonzero. Default is 3.1 (Cardelli et al. 1989, ApJ, 345, 245).")
parser.add_argument("-iq", type=int, metavar="int", help="Image quality. It can be set to \"20\" (0.6\"), \"70\" (0.85\"), \"85\" (1.1\") or \"100\" (1.9\")")
parser.add_argument("-cc", type=int, metavar="int", help="Cloud cover. It can be set to \"50\" (no extinction), \"70\" (0.3mag extinction), \"80\" (1mag extinction) or \"100\" (3mag extinction).")
parser.add_argument("-sb", type=int, metavar="int", help="Sky brightness. It can be set to \"20\" (darkest), \"50\" (dark), \"80\" (grey) or \"100\" (bright).")
parser.add_argument("-airmass", type=float, metavar="float", help="Airmass. The value needs to be between 1 and 3. Note that we rarely observe at airmasses higher than 2, unless it is requested.")
parser.add_argument("-rdmode", type=str, metavar="str", help="Read mode. It can be \"slow\" (2.5 e-/pix) or \"normal\" (3.5 e-/pix).")
parser.add_argument("-spmode", type=int, metavar="int", help="Spectral mode. It can be \"1\" fiber (R=67.5k) or \"2\" fibers (R=40k).")
parser.add_argument("-nexp", type=int, metavar="int", help="Number of exposures. Default is 1.")
parser.add_argument("-exptime", type=int, metavar="int", help="Exposure time in seconds. Note that GRACES exposure time can only be adjusted as an integer number of seconds, with a minimum of 1s.")

args = parser.parse_args()
dirin = args.dirin
dirout = args.dirout
name = args.name
objtype = args.objtype
fwhm = args.fwhm
templ = args.templ
ctempl = args.ctempl
temp = args.temp
expn = args.expn
mag = args.mag
filter = args.filter
z = args.z
av = args.av
rv = args.rv
iq = args.iq
cc = args.cc
sb = args.sb
airmass = args.airmass
rdmode = args.rdmode
spmode = args.spmode
nexp = args.nexp
exptime = args.exptime

# Requesting missing inputs
# Path to "templates" and "filters" folders
if dirin == None:
	dirin = "."
while os.path.exists(dirin+"/filters")+os.path.exists(dirin+"/templates") != 2:
	print("\nThe folders \"filters\" and \"templates\" could not be found in the directory \""+dirin+"\".")
	dirin = input("Please try another directory: ")

# Path for output files
if dirout == None:
	dirout = "."
while os.path.exists(dirout) == False:
	print("\nOutput directory \""+dirout+"\" does not exist.")
	dirout = input("Please try another directory: ")

# Basename for output files
if name == None:
	name = "ITC"

# Object type
lst = ["pointsource", "gaussian", "uniform"]
ok = 0
if objtype != None:
	if objtype.lower() in lst:
		objtype = objtype.lower()
		ok = 1
	else:
		print("\nObject type \""+objtype+"\" is not a valid option.")
else:
	objtype = "pointsource"
	ok = 1
if ok == 0:
	print("Which object type would you like to choose?")
	print(" (1) point source")
	print(" (2) gaussian")
	print(" (3) uniform")
	number = 0
	while number<1 or number>3:
		number = int(input("Enter the number corresponding to the desired object type: "))
	if number == 1: objtype = "pointsource"
	if number == 2: objtype = "gaussian"
	if number == 3: objtype = "uniform"

# FWHM of the Gaussian brighness profile
if fwhm == None and objtype == "gaussian":
	fwhm = float(input("\nWhat is the FWHM of the Gaussian brightness profile in arcsec? "))

# Spectral template
lst = np.loadtxt(dirin+"/templates/list.txt", dtype="str")
ok = 0
if templ != None:
	if templ.lower()+".txt" in lst or templ.lower()=="blackbody" or templ.lower()=="powerlaw" or templ.lower() == "custom":
		templ = templ.lower()
		ok = 1
	else:
		print("\nTemplate \""+templ+"\" is not recognised.")
if ok == 0:
	if templ == None: print("")
	print("Please choose a spectral template:")
	for i in range(len(lst)):
		print(" ("+str(i+1)+") "+lst[i][:-4])
	print(" ("+str(len(lst)+1)+") blackbody")
	print(" ("+str(len(lst)+2)+") powerlaw")
	print(" ("+str(len(lst)+3)+") custom")
	number = 0
	while number<1 or number>len(lst)+3:
		number = int(input("Enter the number corresponding to the desired template: "))
	if number <= len(lst): templ = lst[number-1][:-4]
	if number == len(lst)+1: templ = "blackbody"
	if number == len(lst)+2: templ = "powerlaw"
	if number == len(lst)+3: templ = "custom"

# Blackbody temperature
if templ=="blackbody":
	if temp == None:
		temp = float(input("\nEnter the black body temperature (K): "))
	hconst = 6.626070e-34 #J*s
	kconst = 1.380649e-23 #J/K
	c = 299792458 #m/s

# Power law exponent
if templ=="powerlaw":
	if expn == None:
		expn = float(input("\nEnter the power law exponent: "))

# Custom template filename
if templ=="custom":
	if ctempl == None:
		ctempl = str(input("\nEnter the (path and) filename of your custom spectral template: "))
	while os.path.exists(ctempl) == False:
		print("\nCustom template \""+ctempl+"\" does not exist.")
		ctempl = input("Please try a different (path and) filename: ")

# Magnitude
if mag == None and objtype != "uniform":
	mag = float(input("\nWhat magnitude is the target? "))
if mag == None and objtype == "uniform":
	mag = float(input("\nWhat is the target's surface brightness in mag/arcsec^2? "))

# Filter
lst = os.listdir(dirin+"/filters")
lst.sort()
ok = 0
if filter != None:
	if any(filter in s[1] for s in lst):
		ok = 1
	else:
		print("\nFilter \""+filter+"\" is not recognised.")
if ok == 0:
	if filter == None: print("")
	print("Which filter does the target magnitude correspond to?")
	for i in range(len(lst)):
		print(" ("+str(i+1)+") "+lst[i][1])
	number = 0
	while number<1 or number>len(lst):
		number = int(input("Enter the number of the corresponding filter: "))
	filter = lst[number-1][1]
filter = [s for s in lst if filter in s][0]

# Image quality
lst = [20,70,85,100]
ok = 0
if iq != None:
	if iq in lst:
		ok = 1
	else:
		print("\nImage quality \""+str(iq)+"\" is not a valid option.")
if ok == 0:
	if iq == None: print("")
	print("Which image quality would you like to use?")
	print(" (1) 20%-ile")
	print(" (2) 70%-ile")
	print(" (3) 85%-ile")
	print(" (4) 100%-ile")
	number = 0
	while number<1 or number>4:
		number = int(input("Enter the number corresponding to the desired image quality: "))
	if number == 1: iq = 20
	if number == 2: iq = 70
	if number == 3: iq = 85
	if number == 4: iq = 100

# Cloud cover
lst = [50,70,80,100]
ok = 0
if cc != None:
	if cc in lst:
		ok = 1
	else:
		print("\nCloud cover \""+str(cc)+"\" is not a valid option.")
if ok == 0:
	if cc == None: print("")
	print("Which cloud cover would you like to use?")
	print(" (1) 50%-ile")
	print(" (2) 70%-ile")
	print(" (3) 80%-ile")
	print(" (4) 100%-ile")
	number = 0
	while number<1 or number>4:
		number = int(input("Enter the number corresponding to the desired cloud cover: "))
	if number == 1: cc = 50
	if number == 2: cc = 70
	if number == 3: cc = 80
	if number == 4: cc = 100

# Sky background
lst = [20,50,80,100]
ok = 0
if sb != None:
	if sb in lst:
		ok = 1
	else:
		print("\nSky brightness \""+str(sb)+"\" is not a valid option.")
if ok == 0:
	if sb == None: print("")
	print("Which sky brightness would you like to use?")
	print(" (1) 20%-ile")
	print(" (2) 50%-ile")
	print(" (3) 80%-ile")
	print(" (4) 100%-ile")
	number = 0
	while number<1 or number>4:
		number = int(input("Enter the number corresponding to the desired sky brightness: "))
	if number == 1: sb = 20
	if number == 2: sb = 50
	if number == 3: sb = 80
	if number == 4: sb = 100

# Airmass
ok = 0
if airmass != None:
	if airmass>=1 and airmass<=3:
		ok = 1
	else:
		print("\nAirmass \""+str(airmass)+"\" is out of acceptable range.")
if ok == 0:
	if airmass == None: print("")
	airmass = 0
	while airmass<1 or airmass>3:
		airmass = float(input("What airmass would you like to use [1-3]? "))

# Read mode
lst = ["slow","normal"]
ok = 0
if rdmode != None:
	if rdmode.lower() in lst:
		rdmode = rdmode.lower()
		ok = 1
	else:
		print("\nRead mode \""+rdmode+"\" is not a valid option.")
if ok == 0:
	if rdmode == None: print("")
	print("Which read mode would you like to use?")
	print(" (1) slow (2.5 e-/pix)")
	print(" (2) normal (3.5 e-/pix)")
	number = 0
	while number<1 or number>2:
		number = int(input("Enter the number corresponding to the desired read mode: "))
	if number == 1: rdmode = "slow"
	if number == 2: rdmode = "normal"
if rdmode == "slow":
	rdnoise = 2.5
	gain = 1.02
	readtime = 36
if rdmode == "normal":
	rdnoise = 3.5
	gain = 1.185
	readtime = 25

# Spectral mode
lst = [1,2]
ok = 0
if spmode != None:
	if spmode in lst:
		ok = 1
	else:
		print("\nSpectral mode \""+spmode+"\" is not a valid option.")
if ok == 0:
	if spmode == None: print("")
	print("Which spectral mode would you like to use?")
	print(" (1) 1 fiber (target only, R=67.5k)")
	print(" (2) 2 fibers (target+sky, R=40k)")
	number = 0
	while number<1 or number>2:
		number = int(input("Enter the number corresponding to the desired spectral mode: "))
	if number == 1: spmode = 1
	if number == 2: spmode = 2

# Number of exposures
if nexp == None:
	nexp = 1

# Exposure time
if exptime == None:
	exptime = int(input("\nWhat exposure time would you like to use [s]? "))

# Display selected parameters
print("\n!!!*** Please copy paste the following in the ITC example section: ***!!!\n")
print("Processed with GRACES ITC v2.1.")
if objtype == "pointsource": info = " POINT SOURCE"
if objtype == "gaussian": info = " GAUSSIAN SOURCE with FWHM = "+str(fwhm)+"\""
if objtype == "uniform": info = " UNIFORM SOURCE"
if templ == "blackbody": info = info + "\n BLACKBODY with T = "+str(round(temp))+" K"
elif templ == "powerlaw": info = info + "\n POWER-LAW with exponent = "+str(expn)
elif templ == "custom": info = info + "\n CUSTOM spectrum"
else: info = info + "\n "+templ.upper()+" library spectrum"
info = info + "\n "+filter[1]+" = "+str(mag)+" mag"
if objtype == "uniform": info = info + "/arcsec^2"
if z != None and z != 0: info = info + "\n z = "+str(z)
if rv == None:
	rv = 3.1
if av != None and av != 0: info = info + "\n Av = "+str(av)+" with Rv = "+str(rv)
info = info + "\n IQ"+str(iq)+"% CC"+str(cc)+"% SB"+str(sb)+"%"
info = info + "\n airmass = "+str(airmass)
info = info + "\n "+rdmode.upper()+" read mode"
info = info + "\n "+str(spmode)+"-fiber mode"
infofig = info
info = info + "\n setup time = 600s"
if nexp == 1:
	info = info + "\n exposure time = "+str(exptime)+"s"
	infofig = infofig + "\n exposure time = "+str(exptime)+"s"
else:
	info = info + "\n exposure time = "+str(nexp)+"\u00D7"+str(exptime)+"s"
	infofig = infofig + "\n exposure time = "+str(nexp)+"\u00D7"+str(exptime)+"s"
if nexp == 1: info = info + "\n readout time = "+str(readtime)+"s"
else: info = info + "\n readout time = "+str(nexp)+"\u00D7"+str(readtime)+"s"
nreacq = np.ceil(nexp*(exptime+readtime)/7200)-1
if nexp>1 and nreacq == 1:
	info = info + "\n reacquisition time = 300s"
if nexp>1 and nreacq > 1:
	info = info + "\n reacquisition time = "+str(int(nreacq))+"\u00D7300s"
info = info + "\n total time = "+str(round((600+nexp*(exptime+readtime)+nreacq*300)/60,1))+" minutes"
print(info)

# Long exposure time warning
if exptime > 2400:
	nexp_new = int(nexp*exptime/2400)
	if (nexp*exptime) % 2400 != 0:
		nexp_new = nexp_new + 1
	print("\n!!! WARNING !!! Recommended maximum exposure time is 2400s per spectrum. We therefore suggest to split your total exposure time into "+str(nexp_new)+" spectra of "+str(round(nexp*exptime/nexp_new))+"s. But, if needed, the exposure time can be set to as high as 7200s.")

# Wavelength axis for the template spectrum
b1 = 3000
b2 = 10500
dl = 1/100
xint = np.linspace(b1, b2, int((b2-b1)/dl)+1)
ltmpl = xint/10

# Prepare the template spectrum
if templ == "blackbody":
	itmpl = 2*hconst*c**2/(ltmpl/1e9)**5*1/(np.exp(hconst*c/((ltmpl/1e9)*kconst*temp))-1)
elif templ == "powerlaw":
	itmpl = ltmpl**expn
elif templ == "custom":
	ltmpl,itmpl = np.loadtxt(ctempl, dtype="float", unpack=True)
else:
	ltmpl,itmpl = np.loadtxt(dirin+"/templates/"+templ+".txt", dtype="float", unpack=True)
ltmpl = ltmpl*10
if z != None and z != 0:
	lsh = (z+1)*ltmpl
	dlt = ltmpl[1]-ltmpl[0]
	ltmpl = np.linspace(b1, max(lsh), int((max(lsh)-b1)/dlt)+1)
	tmp = interp1d(lsh, itmpl, kind="linear", fill_value="extrapolate")(ltmpl)
	pos = np.where(ltmpl < min(lsh))
	tmp[pos] = 0
	itmpl = tmp
tmpl = interp1d(ltmpl, itmpl, kind="linear", fill_value="extrapolate")(xint)
pos = np.where(xint < min(ltmpl))
tmpl[pos] = itmpl[0]
pos = np.where(xint > max(ltmpl))
tmpl[pos] = itmpl[-1]

# Apply target magnitude and filter to the spectral template
lfl,ifl = np.loadtxt(dirin+"/filters/"+filter, dtype="float", unpack=True)
lfl = lfl*10
pos = np.where((xint >= min(lfl)) & (xint <= max(lfl)))
cfxint = xint[pos]
cftmpl = tmpl[pos]
transm = interp1d(lfl, ifl, kind="linear", fill_value="extrapolate")(cfxint)
conv = cftmpl*transm
relflux = sum(conv)*dl
if filter[1] == "U": m0 = 14.90
if filter[1] == "B": m0 = 15.80
if filter[1] == "V": m0 = 15.17
if filter[1] == "R": m0 = 15.22
if filter[1] == "I": m0 = 14.50
if filter[1] == "u": m0 = 15.38
if filter[1] == "g": m0 = 15.92
if filter[1] == "r": m0 = 15.23
if filter[1] == "i": m0 = 14.91
if objtype != "uniform":
	fac = 10**(-0.4*(mag-m0))/relflux
if objtype == "uniform":
	fiber = 1.2
	mag_fiber = mag - 2.5*np.log10(np.pi*(fiber/2)**2)
	fac = 10**(-0.4*(mag_fiber-m0))/relflux
itmpl = itmpl*fac

# Apply IQ and Gaussian FWHM losses
if iq == 20: seeing = 0.6
if iq == 70: seeing = 0.85
if iq == 85: seeing = 1.1
if iq == 100: seeing = 1.9
fiber = 1.2
if objtype == "pointsource":
	frac = 1-np.exp(-(fiber/2)**2/(seeing**2/(4*np.log(2))))
if objtype == "gaussian":
	frac = 1-np.exp(-(fiber/2)**2/((seeing**2+fwhm**2)/(4*np.log(2))))
if objtype == "uniform":
	frac = 1
itmpl = itmpl*frac

# Apply CC losses
if cc == 50: ext = 0
if cc == 70: ext = 0.3
if cc == 80: ext = 1
if cc == 100: ext = 3
frac = 10**(-0.4*ext)
itmpl = itmpl*frac

# Airmass extinction at Mauna Kea
extl = np.array([310, 320, 340, 360, 380, 400, 450, 500, 550, 600, 650, 700, 800, 900, 1100, 1200, 1250])*10
extv = [1.37, 0.82, 0.51, 0.37, 0.30, 0.25, 0.17, 0.13, 0.12, 0.11, 0.11, 0.10, 0.07, 0.05, 0.02, 0.017, 0.015]
ext = interp1d(extl, extv, kind="linear", fill_value="extrapolate")(ltmpl)
frac = 10**(-0.4*ext*(airmass-1))
itmpl = itmpl*frac

# Clip template between 350nm and 1250nm, a more reasonable interval for GRACES
pos = np.where((ltmpl >= 3500) & (ltmpl <= 12500))
ltmpl = ltmpl[pos]
itmpl = itmpl[pos]

# Interstellar extinction and reddening
if av != None and av != 0:
	yy = 1/(ltmpl/10000)-1.82
	ax = 1 + 0.17699*yy - 0.50447*yy**2 - 0.02427*yy**3 + 0.72085*yy**4 + 0.01979*yy**5 - 0.7753*yy**6 + 0.32999*yy**7
	bx = 1.41338*yy + 2.28305*yy**2 + 1.07233*yy**3 - 5.38434*yy**4 - 0.62251*yy**5 + 5.3026*yy**6 - 2.09002*yy**7
	Albd = (ax+bx/rv)*av
	itmpl = itmpl*10**(-0.4*Albd)

# Calibration function, originally based on Feige 66 (2015) and later refined
if spmode == 1:
	xint = np.linspace(4027.93, 10482.4, int((10482.4-4027.93)/0.0500765)+1)
if spmode == 2:
	xint = np.linspace(4027.93, 10482.4, int((10482.4-4027.93)/0.0507818)+1)
x_courbe = [4000, 4150, 4500, 5000, 5250, 6000, 6500, 7000, 8000, 9000, 10000, 10500]
y_courbe = [15, 45, 216, 684, 969, 1824, 2736, 3876, 5700, 4560, 1254, 285]
courbe = interp1d(x_courbe, y_courbe, kind="cubic")(xint)

# Interpolate template spectrum
tmp = interp1d(ltmpl, itmpl, kind="linear", fill_value="extrapolate")(xint)
tmpl = tmp

# Sky spectrum (e-/s/pix)
sky20 = -5.23206 + 0.00281598*xint - 4.47676e-07*xint**2 + 7.82084e-12*xint**3 + 3.74855e-15*xint**4 - 2.29490e-19*xint**5
sky50r = 7.10159 - 0.00428956*xint + 1.25479e-06*xint**2 - 1.71528e-10*xint**3 + 1.07836e-14*xint**4 - 2.50147e-19*xint**5
sky80r = 18.2547 - 0.0117544*xint + 3.47604e-06*xint**2 - 4.88248e-10*xint**3 +3.17787e-14*xint**4 - 7.69180e-19*xint**5
sky100r = 61.4096 - 0.0407968*xint + 1.21107e-05*xint**2 - 1.72026e-09*xint**3 + 1.13804e-13*xint**4 - 2.81462e-18*xint**5
if spmode == 1: sky20 = sky20/107
if spmode == 2: sky20 = sky20/63
if sb == 20: sky = sky20
if sb == 50: sky = sky20*sky50r
if sb == 80: sky = sky20*sky80r
if sb == 100: sky = sky20*sky100r
Nst = sky*exptime

# Aperture size in pixels
if spmode == 1: apert = 22
if spmode == 2: apert = 12

# Signal per exposure
signal = tmpl*courbe*exptime

# Check for saturation
nlt = 46400*gain #non-linearity threshold in e-
peak = max(signal/apert+Nst)
if peak >= nlt:
	if 1/exptime*peak > nlt:
		print("\n!!! WARNING !!! This spectrum will be saturated even with the shortest possible exposure time of 1s.")
	else:
		nexp_new = int(np.ceil(nexp*peak/(nlt*2/3)))
		exptime_new = int(np.round(nexp*exptime/nexp_new))
		if exptime_new/exptime*peak > nlt*2/3:
			exptime_new = int(np.floor(nexp*exptime/nexp_new))
		print("\n!!! WARNING !!! It is recommended to split your total exposure time into "+str(nexp_new)+" spectra of "+str(exptime_new)+"s each to avoid saturation.")

# Signal to noise ratio
SN = nexp*signal/np.sqrt(nexp*signal+nexp*apert*Nst+nexp*apert*rdnoise**2)

# Plotting
plt.figure(figsize=[8.5,5])
plt.xlim([400,1040])
plt.ylim(ymin=0, ymax=max(SN)*1.13)
plt.xlabel("Wavelength [nm]")
plt.ylabel("S/N [pix$^{-1}$]")
plt.text(407, 1.1*max(SN), infofig, fontsize=8, color=(0.4,0.4,0.4), verticalalignment="top", zorder=3)
plt.grid(linewidth=0.3, zorder=1)
plt.plot(xint/10, SN, zorder=2)
plt.tight_layout()
plt.savefig(dirout+"/"+name+".png")
plt.show()

# Write S/N vs. wavelength to a file
infofile = "# Processed with GRACES ITC v2.1.\n"
for line in info.splitlines():
	infofile = infofile + "# " + line + "\n"
infofile = infofile + "#\n"
f = open(dirout+"/"+name+".txt","w")
f.write(infofile)
f.write("# Wavelength_[nm] S/N_[1/pix]\n")
for i in range(len(xint)):
	f.write("%f %f\n" % (xint[i]/10, SN[i]))
f.close()